CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  vga_base_pin = 16
  mouse_data_pin = 24
  mouse_clock_pin = 25

  cols = vgatext#cols 'number of screen columns
  rows = vgatext#rows 'number of screen rows
  chrs = cols * rows 'number of screen character locations

  keyboard  = 26 '' I/O for keyboard connection   
OBJ

  vgatext : "vga_hires_text"
  mouse   : "mouse"
  key     : "comboKeyboard"

VAR

  long  sync
  long  screen[cols * rows / 4] 'screen buffer
  word  colors[rows] 'row colors
  byte  cx0, cy0, cm0, cx1, cy1, cm1 'cursor control bytes
  byte  gencogID 'generator cog ID

  'shared screen, cell work buffer, and counter addresses - must be contiguous and in this order
  long  screenaddr

  'program vars
  long mousex 
  long mousey

  long brushSize

PUB start | i, j, keyval

  'Start mouse
  mouse.start(mouse_data_pin, mouse_clock_pin)
  mouse.bound_limits(0, 0, 0, cols - 1, rows - 1, 0)
  mouse.bound_scales(10, -10, 0)

  'Start keyboard
  key.start(keyboard)

  'stop generator cog
  stopgen

  'set up to pass shared variable addresses to generator cog
  screenaddr := @screen
  
  'set cursor mode
  cm0 := %001

  'start vga text driver
  vgatext.start(vga_base_pin, @screen, @colors, @cx0, @sync)

  'set up row colors
  repeat i from 0 to rows - 1
    colors[i] := %%0020_3333 'set all rows to white foreground, blue background
      
  'fill screen with space character
  bytefill(@screen, $20, chrs)
  
  'Intro
  displaytext(1, 1, string("PropPaint v0.7"), true)
  displaytext(1, 2, string("Created by Owen Trueblood"), true)
  
  displaytext(1, 4, string("Instructions:"), false)
  displaytext(1, 4, string("*Use the mouse to move the cursor around."), false)
  displaytext(1, 5, string("*Use the left mouse button to paint."), false)
  displaytext(1, 6, string("*Use the right mouse button to erase."), false)
  displaytext(1, 7, string("*Use the keyboard keys 1-3 to set the brush size."), false)
  displaytext(1, 8, string("*Hit enter to clear the screen."), false)

  displaytext(1, 10, string("Press any key to continue."), false)    

  'Wait until the user does something
  repeat while keyval == 0 and mouse.button(0)==false and mouse.button(1)==false
    'get keypresses
    keyval := key.key

  'fill screen with space character
  bytefill(@screen, $20, chrs)

  brushSize := 0
  
  'Main loop
  repeat
    'cursor always follows mouse
    cx0 := mouse.bound_x
    cy0 := mouse.bound_y
  
    mousex := mouse.bound_x
    mousey := mouse.bound_y

    'get keypresses
    keyval := key.key
        
    'Reboot if esc is pressed
    if keyval == 203
      reboot

    'left mouse button pressed
    if mouse.button(0)
      paint(mousex, mousey, brushSize)

    'right mouse button pressed
    if mouse.button(1)
      erase(mousex, mousey, brushSize)

    'use keyboard to set brush size
    if keyval > 47 and keyval < 52
      brushSize := keyval-49

    'enter pressed
    if keyval == 13
      bytefill(@screen, $20, chrs)

PRI paint(x, y, size)

  if size==0
    displaytext(x, y, string("X"), false)
  if size==1
    displaytext(x, y, string("X"), false)
    displaytext(x, y-1, string("X"), false)
    displaytext(x, y+1, string("X"), false)
    displaytext(x-1, y, string("X"), false)
    displaytext(x+1, y, string("X"), false)
  if size==2
    displaytext(x, y, string("X"), false)
    displaytext(x, y-1, string("X"), false)
    displaytext(x, y+1, string("X"), false)
    displaytext(x-1, y, string("X"), false)
    displaytext(x+1, y, string("X"), false)
    displaytext(x-1, y-1, string("X"), false)
    displaytext(x-1, y+1, string("X"), false)
    displaytext(x+1, y-1, string("X"), false)
    displaytext(x+1, y+1, string("X"), false)

PRI erase(x, y, size)

  if size==0
    displaytext(x, y, string(" "), false)
  if size==1
    displaytext(x, y, string(" "), false)
    displaytext(x, y-1, string(" "), false)
    displaytext(x, y+1, string(" "), false)
    displaytext(x-1, y, string(" "), false)
    displaytext(x+1, y, string(" "), false)
  if size==2
    displaytext(x, y, string(" "), false)
    displaytext(x, y-1, string(" "), false)
    displaytext(x, y+1, string(" "), false)
    displaytext(x-1, y, string(" "), false)
    displaytext(x+1, y, string(" "), false)
    displaytext(x-1, y-1, string(" "), false)
    displaytext(x-1, y+1, string(" "), false)
    displaytext(x+1, y-1, string(" "), false)
    displaytext(x+1, y+1, string(" "), false)
    
PRI displaytext(col, row, textptr, invert) | pos, len, i, j

  pos := cols * row + col
  len := strsize(textptr)
  repeat i from 1 to len
    j := byte[textptr++]
    if invert
      j |= $80
    screen.byte[pos++] := j

PRI startgen

  ifnot gencogID
    gencogID := cognew(@_generate, @screenaddr) + 1       

PRI stopgen

  if gencogID
    cogstop(gencogID~ - 1) 

DAT 'assembly code for generator cog

                        org     0
_generate               mov     addr, par 'get shared variable addresses
                        rdlong  scrnaddr, addr

'shared variable addresses
addr                    res     1
scrnaddr                res     1